# -*- coding: utf-8 -*-

#Copyright 2009 tv-player team <tvplayer.staff@gmail.com>
#       
#This program is free software; you can redistribute it and/or modify
#it under the terms of the GNU Lesser General Public License as published by
#the Free Software Foundation; version 3 of the License.
#       
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU Lesser General Public License for more details.
#       
#You should have received a copy of the GNU Lesser General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#MA 02110-1301, USA.

"""
Il modulo contiene una classe in grado di scaricare e auto-generare la lista di
radio e tv partendo da un file remoto.
"""

import urllib, urllib2
import os
import re,random

class ListGenerator() :
    """
    Questa classe si occupa del download e della generazione
    dei dizionari contenenti canali Tv e Radio. Queste informazioni
    vengono salvate in una cartella di configurazione all'interno
    della HOME dell'utente.
    """
    
    URL_TV = "http://tvplayersite.altervista.org/online/tv.list"
    URL_RADIO = "http://tvplayersite.altervista.org/online/radio.list"

    URL_DATABASE = "http://tvplayersite.altervista.org/online/users.list"
    URL_ERROR_IMAGE = "http://tvplayersite.altervista.org/online/error.png"
    
    def __init__(self,config_directory="tvplayer") :
        home = os.path.expanduser('~')
        
        #controlla se esiste il file con la vecchia configurazione     
        if (os.path.exists(home +'/.tvplayer/personal_tv.list')) :
            file = open(home +'/.tvplayer/personal_tv.list', 'r')
            line = file.readline()
            file.close()
            tmp = line.split('#')
            if len(tmp)==2:
				os.system('rm -R $HOME/.tvplayer')
		##
        if not os.path.exists(home +'/.tvplayer'):
            os.makedirs(home +'/.tvplayer')
        if not os.path.exists(home +'/.tvplayer/la7image'):
            os.makedirs(home +'/.tvplayer/la7image')
        if not os.path.exists(home +'/.tvplayer/youtube'):
            os.makedirs(home +'/.tvplayer/youtube')
        if not os.path.exists(home +'/.tvplayer/la7image/cult'):
            os.makedirs(home +'/.tvplayer/la7image/cult')
        self.config_dir = home +'/.tvplayer'
        if (os.path.exists(self.config_dir + '/personal_tv.list'))==False :
            personal=open(os.path.expanduser(self.config_dir + '/personal_tv.list'), 'w')
            personal.close()
        if (os.path.exists(self.config_dir + '/personal_radio.list'))==False :
            personal=open(os.path.expanduser(self.config_dir + '/personal_radio.list'), 'w')
            personal.close()
        if (os.path.exists(self.config_dir + '/error.png'))==False :
            errorfile = urllib.urlopen(self.URL_ERROR_IMAGE)
            localFile = open(os.path.expanduser(self.config_dir + '/error.png'), 'w+')
            localFile.write(errorfile.read())
            errorfile.close()
            localFile.close()
            
        urllib.urlretrieve("http://tvplayersite.altervista.org/online/rai.py",self.config_dir+"/rai.py")

        
    def is_updated(self) :
        """
        La funzione verifica se sono già presenti file di Tv o Radio
        nella cartella di configurazione del programma.
        """
        if os.path.exists(self.config_dir + '/tv.list') :
            if os.path.exists(self.config_dir + '/radio.list') :
                self.primoAvvio()
                return True
        return False

    def default_player(self) :
        """
        La funzione verifica se è già presente il file di configurazione
        nella cartella di configurazione del programma, se si legge il player scelto.
        """
        if (os.path.isfile(self.config_dir + '/tv1-0.config'))==False:
            print "Creo il file di configurazione"
            config=open(os.path.expanduser(self.config_dir + '/tv1-0.config'), 'w')
            config.write("PLAYER#integrato#\nTAB#True#True\nLANGUAGE#0#")
            config.close()
            return ["integrato","\n","True","True","0",""]
        else:
            config=open(os.path.expanduser(self.config_dir + '/tv1-0.config'),"r")
            player_config=config.readlines()
            tmp=[]
            for i in player_config:
				line=i.split('#')
				if line[0]=='PLAYER' or line[0]=='TAB' or line[0]=='LANGUAGE':
					tmp.append(line[1].replace('\n',''))
					tmp.append(line[2].replace('\n',''))
            config.close()
            return tmp

    def write_player_config(self, data, data2,data3,data4,data5):
        if data2=='\n':
            data2=''
        config=open(os.path.expanduser(self.config_dir + '/tv1-0.config'), 'w')
        config.write('PLAYER#'+data+"#"+data2+'\nTAB#'+data3+"#"+data4+'\nLANGUAGE#'+data5+'#')
        config.close()
        
    def get_rai_link(self, url):     
        #req = urllib2.Request(url, None, {'User-agent': "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.1.3) Gecko/20090910 Firefox/3.5.3",'viaurl': 'www.rai.tv','ttAuth': token,'Connection': 'keep-alive','Keep-Alive' : '300'})
       # fp = urllib2.urlopen(req)
       # urldirect = fp.read()
       # fp.close()
       # tmp=urldirect.split('"')
      #  print tmp[3]
      #  return tmp[3]
        os.system("python "+self.config_dir+"/rai.py "+url)
        fp=open(self.config_dir+"/raistream.tmp","r")
        url=fp.read()
        fp.close()
        return url

    def get_rairec_link(self, url):
       
        if url[:3]!='mms': 
            req = urllib2.Request(url, None, {'User-agent': "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.1.3) Gecko/20090910 Firefox/3.5.3",'viaurl': 'www.rai.tv'})
            fp = urllib2.urlopen(req)
            urldirect = fp.read()
            fp.close()
            tmp=urldirect.split('"')
            url = tmp[3].replace('mms','http')
        
        url = url.replace('mms','http')

        req = urllib2.Request(url, None, {'User-agent': "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.1.3) Gecko/20090910 Firefox/3.5.3",'viaurl': 'www.rai.tv'})
        fp = urllib2.urlopen(req)
        urldirect = fp.readlines()
        print urldirect[1][5:-2]
        fp.close()
        return urldirect[1][5:-2]
        
    def get_flv_video_url(self,url):
        flv_url = ''
        doc = urllib2.urlopen(url)
        data = doc.read()
        doc.close()
        video_name_re = re.compile(r', "t": "([^"]+)"')
        match = video_name_re.search(data)
        if match is not None:
            video_name = match.group(1)

            url_splited = url.split("watch?v=")
            video_id = url_splited[1]     

            flv_url = "http://www.youtube.com/get_video?video_id=%s&t=%s"
            flv_url = flv_url % (video_id, video_name)
        return flv_url
       
    
    def download(self) :
        """
        Scarica i file contenenti le liste di canali.
        """
           
        # Fetching TV.
        tvwebfile = urllib.urlopen(self.URL_TV)
        localFile = open(os.path.expanduser(self.config_dir + '/tv.list'), 'w+')
        localFile.write(tvwebfile.read())
        tvwebfile.close()
        localFile.close()
        
        # Fetching Radio.
        radiowebfile = urllib.urlopen(self.URL_RADIO)
        localFile = open(os.path.expanduser(self.config_dir + '/radio.list'), 'w+')
        localFile.write(radiowebfile.read())
        radiowebfile.close()
        localFile.close()
        self.primoAvvio()

    def download_DATABASE(self) :
        """
        Scarica il database
        """
        datawebfile = urllib.urlopen(self.URL_DATABASE)
        localFile = open(os.path.expanduser(self.config_dir + '/database.list'), 'w+')
        localFile.write(datawebfile.read())
        datawebfile.close()
        localFile.close()

    def primoAvvio(self) :
        """
        Al primo avvio unisce i programmi di default alla lista personale
        """
        if (os.path.exists(self.config_dir + '/primoavvio'))==False :
            primoavvio=open(os.path.expanduser(self.config_dir + '/primoavvio'), 'w')
            primoavvio.close()
            tvfile = open(self.config_dir +'/tv.list', 'r')
            file=open(self.config_dir + '/personal_tv.list', 'a')
            for line in tvfile.readlines() :
                file.write(line)
            tvfile.close()
            file.close()

            radiofile = open(self.config_dir + '/radio.list', 'r')
            file=open(self.config_dir + '/personal_radio.list', 'a')
            result = {}
            for line in radiofile.readlines() :
                file.write(line)
            radiofile.close()
            file.close()
		
    def generatePersonalList(self,type) :
        """
        Effettua il parsing della lista dei canali Personali per generare
        un dizionario.
        """
        personalfile = open(self.config_dir + '/personal_'+type+'.list', 'r')
        result = []
        for line in personalfile.readlines() :
            result.append(line[:-1])
        personalfile.close()
        return result
        
    def generateTGRList(self) :
        """
        Effettua il parsing della lista dei canali Personali per generare
        un dizionario.
        """
        tgrfile = open('regioni.list', 'r')
        result = {}
        for line in tgrfile.readlines() :
            temp = line.split('#')
            name, addr = temp[0], temp[1]
            result[name] = addr[:-1]
        tgrfile.close()
        return result

    def generateDatabaseList(self) :
        """
        Effettua il parsing della lista dei canali del Database per generare
        un dizionario.
        """
        datafile = open(self.config_dir +'/database.list', 'r')
        result = {}
        for line in datafile.readlines() :
            line = line.replace('\n','')
            line = line.replace('\r','')
            temp = line.split('#')
            name, addr, nation = temp[0], temp[1], temp[3]
            result[name+" #"+temp[2][:-1]+" #"+nation] = addr[:-1]
        datafile.close()
        return result

    def removeitemPersonalList(self,link_remove,type) :
        """
        Rimuove il link scelto dalla lista personale
        """
        personalfile = open(self.config_dir + '/personal_'+type+'.list', 'r')
        result = []
        for line in personalfile.readlines() :
            temp = line.split('#')
            name, addr = temp[0], temp[1]
            if link_remove != addr:
                result.append(name+"#"+addr+"#"+temp[2][:-1])
        personalfile.close()

        personalfile = open(self.config_dir + '/personal_'+type+'.list', 'w')
        for i in result :
            personalfile.write(i+"\n")
        personalfile.close()
        
    def reorderPersonalList(self,new_order,type,language) :
        """
        Riordina la lista personale
        """
        personalfile = open(self.config_dir + '/personal_'+type+'.list', 'r')
        file = []
        newfile = []
        for line in personalfile.readlines():
            file.append(line[:-1])
        personalfile.close()
        cont=0
        scambio=[]
        if language=='ALL':
            newfile=new_order
        else:
            for i in file:
                if cont!=len(new_order):
                    if i.split('#')[2]==new_order[cont].split('#')[2]:
                        newfile.append(new_order[cont])
                        cont+=1
                    else:
						newfile.append(i)
                else:
                    newfile.append(i)
        personalfile = open(self.config_dir + '/personal_'+type+'.list', 'w')
        for i in newfile :
            personalfile.write(i+"\n")
        personalfile.close()
